# -*- coding: utf-8 -*-
import os
import csv
#import unicodecsv
import requests
import requests_cache
import io
import json

from django.conf import settings
from django import db
from django.contrib.gis.gdal import DataSource, SpatialReference, CoordTransform
from django.contrib.gis.geos import GEOSGeometry, MultiPolygon, Point

from munigeo.models import *
from munigeo.importer.sync import ModelSyncher
from munigeo import ocd

from munigeo.importer.base import Importer, register_importer

CITADEL_LIST = [
    {
        'url': 'http://www.citadelonthemove.eu/Portals/0/PropertyAgent/517/Files/11/hospitals.json',
        'cat_map': {
            'hospital': {
                'category': 'hospital',
                'category_desc': 'Hospital',
            }
        }
    }, {
        'url': 'http://www.citadelonthemove.eu/Portals/0/PropertyAgent/517/Files/14/museums-galleries.json',
        'cat_map': {
            'galleries': {
                'category': 'gallery',
                'category_desc': 'Gallery',
            },
            'museums': {
                'category': 'museum',
                'category_desc': 'Museum',
            }
        }
    }, {
        'url': 'http://www.citadelonthemove.eu/Portals/0/PropertyAgent/517/Files/12/hotel_conventioncenter.json',
        'cat_map': {
            'hotels': {
                'category': 'hotel',
                'category_desc': 'Hotel',
            },
            'hotel': {
                'category': 'hotel',
                'category_desc': 'Hotel',
            },
            'convention center': {
                'category': 'convention center',
                'category_desc': 'Convention center',
            },
        }
    }, {
        'url': 'http://www.citadelonthemove.eu/Portals/0/PropertyAgent/517/Files/5/ParkingAthens.json',
        'cat_map': {
            'Parking': {
                'category': 'parking',
                'category_desc': 'Parking lot',
            }
        }
    }
]

def convert_from_wgs84(coords):
    pnt = Point(coords[1], coords[0], srid=4326)
    pnt.transform(PROJECTION_SRID)
    return pnt

@register_importer
class AthensImporter(Importer):
    name = "athens"

    def __init__(self, *args, **kwargs):
        super(AthensImporter, self).__init__(*args, **kwargs)
        self.data_path = self.options['data_path']
        self.muni_data_path = os.path.join(self.data_path, 'gr', 'athens')

    def import_municipalities(self):
        muni, c = Municipality.objects.get_or_create(id=30001, name="Athens")
        self.logger.info("Athens municipality added.")

    def import_pois_from_citadel(self):
        muni = Municipality.objects.get(id=30001)
        for d in CITADEL_LIST:
            self._import_citadel(muni, d)

    def import_pois(self):
        requests_cache.install_cache('geo_import_athens')
        self.logger.info("Importing POIs from Citadel")
        self.import_pois_from_citadel()



# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-11-10 08:46
from __future__ import unicode_literals

import django.contrib.gis.db.models.fields
from django.db import migrations, models
import django.db.models.deletion

from munigeo.utils import get_default_srid
DEFAULT_SRID = get_default_srid()


class Migration(migrations.Migration):

    dependencies = [
        ('munigeo', '0003_add_modified_time_to_address_and_street'),
    ]

    operations = [
        migrations.CreateModel(
            name='Building',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('origin_id', models.CharField(db_index=True, max_length=40)),
                ('geometry', django.contrib.gis.db.models.fields.MultiPolygonField(srid=DEFAULT_SRID)),
                ('modified_at', models.DateTimeField(auto_now=True, help_text='Time when the information was last changed')),
                ('addresses', models.ManyToManyField(blank=True, to='munigeo.Address')),
                ('municipality', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='munigeo.Municipality')),
            ],
            options={
                'ordering': ['municipality', 'origin_id'],
            },
        ),
    ]



"""
munigeo importer for Finnish nation-level data
"""

import re
import os
import zipfile
import requests
import io

from django import db
from django.contrib.gis.gdal import DataSource
from django.contrib.gis.geos import MultiPolygon, Polygon

from munigeo.importer.base import Importer, register_importer
from munigeo.importer.sync import ModelSyncher
from munigeo.models import AdministrativeDivision, AdministrativeDivisionGeometry, AdministrativeDivisionType, \
    Municipality, PROJECTION_SRID
from munigeo import ocd
from .helsinki import FIN_GRID, TM35_SRID

try:
    from concurrent.futures import ThreadPoolExecutor
except ImportError:
    ThreadPoolExecutor = None
# Disable threaded mode for now
ThreadPoolExecutor = None

MUNI_DATA_URL = 'http://kartat.kapsi.fi/files/kuntajako/kuntajako_1000k/etrs89/gml/TietoaKuntajaosta_2016_1000k.zip'


@register_importer
class FinlandImporter(Importer):
    name = "finland"

    def _process_muni(self, syncher, feat):
        muni_id = str(feat.get('nationalCode'))
        t = feat.get('text')
        m = re.match(r'\(2:([\w\s:-]+),([\w\s:-]+)\)', t)
        name_fi = m.groups()[0]
        name_sv = m.groups()[1]
        self.logger.debug(name_fi)

        munidiv = syncher.get(muni_id)
        if not munidiv:
            munidiv = AdministrativeDivision(origin_id=muni_id)
        munidiv.name_fi = name_fi
        munidiv.name_sv = name_sv
        munidiv.ocd_id = ocd.make_id(country='fi', kunta=name_fi)
        munidiv.type = self.muni_type
        munidiv.save()
        syncher.mark(munidiv)

        try:
            geom_obj = munidiv.geometry
        except AdministrativeDivisionGeometry.DoesNotExist:
            geom_obj = AdministrativeDivisionGeometry(division=munidiv)
        geom = feat.geom
        geom.transform(PROJECTION_SRID)
        # Store only the land boundaries
        # geom = geom.geos.intersection(self.land_area)
        geom = geom.geos
        if geom.geom_type == 'Polygon':
            geom = MultiPolygon(geom)
        geom_obj.boundary = geom
        geom_obj.save()

        try:
            muni = Municipality.objects.get(division=munidiv)
        except Municipality.DoesNotExist:
            muni = Municipality(division=munidiv)
        muni.name_fi = name_fi
        muni.name_sv = name_sv
        muni.id = munidiv.ocd_id.split('/')[-1].split(':')[-1]
        muni.save()

    def _setup_land_area(self):
        fin_bbox = Polygon.from_bbox(FIN_GRID)
        fin_bbox.srid = TM35_SRID
        fin_bbox.transform(4326)
        self.logger.debug("Loading global land shape")
        path = self.find_data_file('global/ne_10m_land.shp')
        ds = DataSource(path)
        land = ds[0][0]
        self.land_area = fin_bbox.intersection(land.geom.geos)
        self.land_area.transform(PROJECTION_SRID)

    def load_muni_data(self):
        self.logger.info("Loading Finnish municipalities")
        resp = requests.get(MUNI_DATA_URL)
        with io.BytesIO(resp.content) as f:
            zf = zipfile.ZipFile(f)
            for name in zf.namelist():
                if name.endswith('.xml'):
                    break
            else:
                raise Exception('XML file not found in %s' % MUNI_DATA_URL)
            out_path = os.path.join(self.data_paths[0], 'fi')
            try:
                os.makedirs(out_path)
            except OSError:
                pass
            zf.extract(name, out_path)
            return os.path.join(out_path, name)

    def find_muni_data(self):
        for root_path in self.data_paths:
            base_path = os.path.join(root_path, 'fi')
            if not os.path.exists(base_path):
                os.makedirs(base_path)
            paths = os.listdir(base_path)
            for p in paths:
                if 'Kuntajaosta' in p:
                    break
            else:
                return self.load_muni_data()
            xml_dir = p
            base_path = os.path.join(base_path, xml_dir)
            paths = os.listdir(base_path)
            for p in paths:
                if p.endswith('.xml'):
                    break
            else:
                return self.load_muni_data()
            return os.path.join(base_path, p)

    def import_municipalities(self):
        # self._setup_land_area()

        self.logger.info("Loading municipality boundaries")
        path = self.find_muni_data()
        ds = DataSource(path)
        lyr = ds[0]
        assert lyr.name == "AdministrativeUnit"

        defaults = {'name': 'Municipality'}
        muni_type, _ = AdministrativeDivisionType.objects.get_or_create(type='muni', defaults=defaults)
        self.muni_type = muni_type

        syncher = ModelSyncher(AdministrativeDivision.objects.filter(type=muni_type), lambda obj: obj.origin_id)

        # If running under Python 3, parallelize the heavy lifting.
        if ThreadPoolExecutor:
            executor = ThreadPoolExecutor(max_workers=8)
            futures = []
        else:
            executor = None

        with db.transaction.atomic():
            with AdministrativeDivision.objects.disable_mptt_updates():
                for idx, feat in enumerate(lyr):
                    if feat.get('nationalLevel') != '4thOrder':
                        continue
                    # Process the first in a single-threaded way to catch
                    # possible exceptions early.
                    if executor and idx > 0:
                        futures.append(executor.submit(self._process_muni, syncher, feat))
                    else:
                        self._process_muni(syncher, feat)
                if executor:
                    for f in futures:
                        res = f.result()
                    executor.shutdown()

            AdministrativeDivision.objects.rebuild()



from modeltranslation.translator import translator, TranslationOptions
from munigeo.models import AdministrativeDivision, Municipality, Street

class AdministrativeDivisionTranslationOptions(TranslationOptions):
    fields = ('name',)
translator.register(AdministrativeDivision, AdministrativeDivisionTranslationOptions)

class MunicipalityTranslationOptions(TranslationOptions):
    fields = ('name',)
translator.register(Municipality, MunicipalityTranslationOptions)

class StreetTranslationOptions(TranslationOptions):
    fields = ('name',)
translator.register(Street, StreetTranslationOptions)



import re
import json
from django.db.models import Q
from datetime import datetime
from django.conf import settings
from django.contrib.gis.db import models
from rest_framework import serializers, viewsets, generics
from rest_framework.exceptions import ParseError
from django.contrib.gis.gdal import SRSException, CoordTransform, SpatialReference
from django.contrib.gis.geos import Point, Polygon
try:
    from django.contrib.gis.geos.base import gdal
except ImportError:
    # Django 1.9 onwards
    from django.contrib.gis import gdal
from munigeo.models import AdministrativeDivisionType, AdministrativeDivision,\
    AdministrativeDivisionGeometry, Municipality, Street, Address
from modeltranslation import models as mt_models # workaround for init problem
from modeltranslation.translator import translator, NotRegistered

# Use the GPS coordinate system by default
DEFAULT_SRID = 4326
DATABASE_SRID = getattr(settings, 'PROJECTION_SRID', 4326)
DEFAULT_SRS = SpatialReference(DEFAULT_SRID)

all_views = []
def register_view(klass, name):
    all_views.append({'class': klass, 'name': name})

def poly_from_bbox(bbox_val):
    points = bbox_val.split(',')
    if len(points) != 4:
        raise ParseError("bbox must be in format 'left,bottom,right,top'")
    try:
        points = [float(p) for p in points]
    except ValueError:
        raise ParseError("bbox values must be floating points or integers")
    poly = Polygon.from_bbox(points)
    return poly

def srid_to_srs(srid):
    if not srid:
        srid = DEFAULT_SRID
    try:
        srid = int(srid)
    except ValueError:
        raise ParseError("'srid' must be an integer")
    try:
        srs = SpatialReference(srid)
    except SRSException:
        raise ParseError("SRID %d not found (try 4326 for GPS coordinate system)" % srid)
    return srs

def build_bbox_filter(srs, bbox_val, field_name):
    poly = poly_from_bbox(bbox_val)
    poly.set_srid(srs.srid)

    return {"%s__within" % field_name: poly}

def make_muni_ocd_id(name, rest=None):
    country = getattr(settings, 'DEFAULT_COUNTRY', None)
    muni = getattr(settings, 'DEFAULT_OCD_MUNICIPALITY', None)
    if country and muni:
        s = 'ocd-division/country:%s/%s:%s' % (settings.DEFAULT_COUNTRY, settings.DEFAULT_OCD_MUNICIPALITY, name)
    else:
        s = name
    if rest:
        s += '/' + rest
    return s


class TranslatedModelSerializer(serializers.ModelSerializer):
    def __init__(self, *args, **kwargs):
        super(TranslatedModelSerializer, self).__init__(*args, **kwargs)
        model = self.Meta.model
        try:
            trans_opts = translator.get_options_for_model(model)
        except NotRegistered:
            self.translated_fields = []
            return

        self.translated_fields = trans_opts.fields.keys()
        lang_codes = [x[0] for x in settings.LANGUAGES]
        remove_fields = []
        # Remove the pre-existing data in the bundle.
        for field_name in self.translated_fields:
            for lang in lang_codes:
                key = "%s_%s" % (field_name, lang)
                if key in self.fields:
                    del self.fields[key]
            del self.fields[field_name]
            remove_fields.append(field_name)
        for field_name in remove_fields:
            if field_name in self.fields:
                del self.fields[field_name]

    def to_representation(self, obj):
        ret = super(TranslatedModelSerializer, self).to_representation(obj)
        if obj is None:
            return ret
        return self.translated_fields_to_representation(obj, ret)

    def translated_fields_to_representation(self, obj, ret):
        for field_name in self.translated_fields:
            d = {}
            for lang in [x[0] for x in settings.LANGUAGES]:
                key = "%s_%s" % (field_name, lang)
                val = getattr(obj, key, None)
                if val is None:
                    continue
                d[lang] = val

            # If no text provided, leave the field as null
            for key, val in d.items():
                if val is not None:
                    break
            else:
                d = None
            ret[field_name] = d

        return ret


class MPTTModelSerializer(serializers.ModelSerializer):
    def __init__(self, *args, **kwargs):
        super(MPTTModelSerializer, self).__init__(*args, **kwargs)
        for field_name in 'lft', 'rght', 'tree_id', 'level':
            if field_name in self.fields:
                del self.fields[field_name]


# Maintain a cache of different coordinate transformations for performance
# reasons.
srs_cache = {}
coord_transforms = {}

def geom_to_json(geom, target_srs):
    srs = srs_cache.get(geom.srid, None)
    if not srs:
        srs = geom.srs
        srs_cache[geom.srid] = srs

    if target_srs:
        ct_id = '%s-%s' % (geom.srid, target_srs.srid)
        ct = coord_transforms.get(ct_id, None)
        if not ct:
            ct = CoordTransform(srs, target_srs)
            coord_transforms[ct_id] = ct
    else:
        ct = None

    if ct:
        wkb = geom.wkb
        geom = gdal.OGRGeometry(wkb, srs)
        geom.transform(ct)
        geom_name = geom.geom_name.lower()
    else:
        geom_name = geom.geom_type.lower()

    # Accelerated path for points
    if geom_name == 'point':
        if target_srs.projected:
            digits = 2
        else:
            digits = 7
        coords = [round(n, digits) for n in [geom.x, geom.y]]
        return {'type': 'Point', 'coordinates': coords}

    s = geom.geojson
    return json.loads(s)


class GeoModelSerializer(serializers.ModelSerializer):

    def __init__(self, *args, **kwargs):
        super(GeoModelSerializer, self).__init__(*args, **kwargs)
        model = self.Meta.model
        self.geo_fields = []
        model_fields = [f.name for f in model._meta.fields]
        remove_fields = []
        for field_name in self.fields:
            if field_name not in model_fields:
                continue
            field = model._meta.get_field(field_name)
            if not isinstance(field, models.GeometryField):
                continue
            self.geo_fields.append(field_name)
            remove_fields.append(field_name)
        for field_name in remove_fields:
            del self.fields[field_name]

    def to_representation(self, obj):
        # SRS is deduced in ViewSet and passed from there
        self.srs = self.context.get('srs', DEFAULT_SRS)
        ret = super(GeoModelSerializer, self).to_representation(obj)
        if obj is None:
            return ret
        for field_name in self.geo_fields:
            val = getattr(obj, field_name)
            if val is None:
                ret[field_name] = None
                continue
            ret[field_name] = geom_to_json(val, self.srs)
        return ret


class GeoModelAPIView(generics.GenericAPIView):
    def initial(self, request, *args, **kwargs):
        super(GeoModelAPIView, self).initial(request, *args, **kwargs)
        srid = request.query_params.get('srid', None)
        self.srs = srid_to_srs(srid)

    def get_serializer_context(self):
        ret = super(GeoModelAPIView, self).get_serializer_context()
        ret['srs'] = self.srs
        return ret


class AdministrativeDivisionTypeSerializer(TranslatedModelSerializer):
    class Meta:
        model = AdministrativeDivisionType
        fields = '__all__'


class AdministrativeDivisionTypeViewSet(viewsets.ReadOnlyModelViewSet):
    queryset = AdministrativeDivisionType.objects.all()
    serializer_class = AdministrativeDivisionTypeSerializer

register_view(AdministrativeDivisionTypeViewSet, 'administrative_division_type')


class AdministrativeDivisionSerializer(GeoModelSerializer, TranslatedModelSerializer,
                                       MPTTModelSerializer):
    def to_representation(self, obj):
        ret = super(AdministrativeDivisionSerializer, self).to_representation(obj)
        if not 'request' in self.context:
            return ret
        qparams = self.context['request'].query_params
        if qparams.get('geometry', '').lower() in ('true', '1'):
            geom = obj.geometry.boundary
            ret['boundary'] = geom_to_json(geom, self.srs)
        ret['type'] = obj.type.type
        return ret

    class Meta:
        model = AdministrativeDivision
        fields = '__all__'


def parse_lat_lon(query_params):
    lat = query_params.get('lat', None)
    lon = query_params.get('lon', None)
    if not lat and not lon:
        return None

    if not lat or not lon:
        raise ParseError("you must supply both 'lat' and 'lon'")
    try:
        lat = float(lat)
        lon = float(lon)
    except ValueError:
        raise ParseError("'lat' and 'lon' must be floating point numbers")

    point = Point(lon, lat, srid=DEFAULT_SRID)
    if DEFAULT_SRID != DATABASE_SRID:
        ct = CoordTransform(SpatialReference(DEFAULT_SRID),
                            SpatialReference(DATABASE_SRID))
        point.transform(ct)
    return point


class AdministrativeDivisionViewSet(GeoModelAPIView, viewsets.ReadOnlyModelViewSet):
    queryset = AdministrativeDivision.objects.all()
    serializer_class = AdministrativeDivisionSerializer

    def get_queryset(self):
        queryset = super(AdministrativeDivisionViewSet, self).get_queryset()
        filters = self.request.query_params

        if 'type' in filters:
            types = filters['type'].strip().split(',')
            is_name = False
            for t in types:
                # If the given type is not digits, assume it's a type name
                if not re.match(r'^[\d]+$', t):
                    is_name = True
                    break
            if is_name:
                queryset = queryset.filter(type__type__in=types)
            else:
                queryset = queryset.filter(type__in=types)

        point = parse_lat_lon(filters)
        if point:
            geometries = AdministrativeDivisionGeometry.objects.filter(boundary__contains=point)
            queryset = queryset.filter(geometry__in=geometries).distinct()

        if 'input' in filters:
            queryset = queryset.filter(name__icontains=filters['input'].strip())

        if 'ocd_id' in filters:
            # Divisions can be specified with form:
            # division=helsinki/kaupunginosa:kallio,vantaa/äänestysalue:5
            d_list = filters['ocd_id'].lower().split(',')
            ocd_id_list = []
            for division_path in d_list:
                if division_path.startswith('ocd-division'):
                    muni_ocd_id = division_path
                else:
                    ocd_id_base = r'[\w0-9~_.-]+'
                    match_re = r'(%s)/([\w_-]+):(%s)' % (ocd_id_base, ocd_id_base)
                    m = re.match(match_re, division_path, re.U)
                    if not m:
                        raise ParseError("'ocd_id' must be of form 'muni/type:id'")

                    arr = division_path.split('/')
                    muni_ocd_id = make_muni_ocd_id(arr.pop(0), '/'.join(arr))
                ocd_id_list.append(muni_ocd_id)

            queryset = queryset.filter(ocd_id__in=ocd_id_list)

        if 'geometry' in filters:
            queryset = queryset.select_related('geometry')

        if 'origin_id' in filters:
            queryset = queryset.filter(origin_id=filters['origin_id'])

        if 'date' in filters:
            try:
                date = datetime.strptime(filters['date'], '%Y-%m-%d').date()
            except ValueError:
                raise ParseError('Invalid date. The required format is YYYY-MM-DD.')
            queryset = queryset.filter(
                (Q(start__lte=date) | Q(start__isnull=True)) &
                (Q(end__gte=date)   | Q(end__isnull=True)))

        queryset = queryset.select_related('type')

        return queryset

register_view(AdministrativeDivisionViewSet, 'administrative_division')


class StreetSerializer(TranslatedModelSerializer):
    class Meta:
        model = Street
        fields = '__all__'

LANG_CODES = [x[0] for x in settings.LANGUAGES]


class StreetViewSet(GeoModelAPIView, viewsets.ReadOnlyModelViewSet):
    queryset = Street.objects.all()
    serializer_class = StreetSerializer

    def get_queryset(self):
        queryset = super(StreetViewSet, self).get_queryset()
        default_lang = LANG_CODES[0]
        self.lang_code = self.request.query_params.get('language', default_lang)
        if self.lang_code not in LANG_CODES:
            raise ParseError("Invalid language supplied. Supported languages: %s" %
                             ', '.join([x[0] for x in settings.LANGUAGES]))

        filters = self.request.query_params

        if 'municipality' in filters:
            val = filters['municipality'].lower()
            if val.startswith('ocd-division'):
                ocd_id = val
            else:
                ocd_id = make_muni_ocd_id(val)
            try:
                muni = Municipality.objects.get(division__ocd_id=ocd_id)
            except Municipality.DoesNotExist:
                raise ParseError("municipality with ID '%s' not found" % ocd_id)

            queryset = queryset.filter(municipality=muni)

        if 'input' in filters:
            args = {}
            args['name_%s__istartswith' % self.lang_code] = filters['input'].strip()
            queryset = queryset.filter(**args)

        return queryset

register_view(StreetViewSet, 'street')


class AddressSerializer(GeoModelSerializer):
    # Reverse geocoding
    def to_representation(self, obj):
        ret = super(AddressSerializer, self).to_representation(obj)
        if not ret['number_end']:
            ret['number_end'] = None
        if not ret['letter']:
            ret['letter'] = None
        if hasattr(obj, 'distance'):
            ret['distance'] = obj.distance.m
        ret['street'] = StreetSerializer(obj.street).data
        return ret

    class Meta:
        model = Address
        exclude = ('id', 'street')


class AddressViewSet(GeoModelAPIView, viewsets.ReadOnlyModelViewSet):
    queryset = Address.objects.all()
    serializer_class = AddressSerializer

    def get_queryset(self):
        filters = self.request.query_params

        default_lang = LANG_CODES[0]
        self.lang_code = filters.get('language', default_lang)
        if self.lang_code not in LANG_CODES:
            raise ParseError("Invalid language supplied. Supported languages: %s" %
                             ', '.join([x[0] for x in settings.LANGUAGES]))

        queryset = super(AddressViewSet, self).get_queryset()

        street = filters.get('street', None)
        if street is not None:
            if street[0].isnumeric():
                queryset = queryset.filter(street=street)
            else:
                args = {}
                args['street__name_%s__iexact' % self.lang_code] = street.strip()
                queryset = queryset.filter(**args)

        if 'municipality' in filters:
            val = filters['municipality'].lower()
            if val.startswith('ocd-division'):
                ocd_id = val
            else:
                ocd_id = make_muni_ocd_id(val)
            try:
                muni = Municipality.objects.get(division__ocd_id=ocd_id)
            except Municipality.DoesNotExist:
                raise ParseError("municipality with ID '%s' not found" % ocd_id)

            queryset = queryset.filter(street__municipality=muni)
        elif 'municipality_name' in filters:
            val = filters['municipality_name'].lower()
            args = {}
            args['name_%s__iexact' % self.lang_code] = val
            try:
                muni = Municipality.objects.get(**args)
                queryset = queryset.filter(street__municipality=muni)
            except Municipality.DoesNotExist:
                queryset = queryset.none()

        number = filters.get('number', None)
        if number is not None:
            queryset = queryset.filter(number=number)

        point = parse_lat_lon(self.request.query_params)
        if point:
            queryset = queryset.distance(point).order_by('distance')

        return queryset

register_view(AddressViewSet, 'address')


class MunicipalitySerializer(TranslatedModelSerializer):
    class Meta:
        model = Municipality
        fields = '__all__'



import pytest

from munigeo.models import Municipality
from munigeo.importer.sync import ModelSyncher


@pytest.fixture
def municipalities():
    models = []
    for x in range(10):
        m = Municipality(
            id=str(x),
            name=str(x),
            division=None)
        models.append(m)
    return models


@pytest.fixture
def syncher(municipalities):
    syncher = ModelSyncher(municipalities, lambda x: x.id)
    for i, m in enumerate(syncher.obj_dict.values()):
        if i % 3 == 0:
            m._found = True
        else:
            m._found = False
    return syncher


def test_syncher_deleted_objects(syncher):
    deleted_objects = syncher.get_deleted_objects()
    assert len(deleted_objects) < len(syncher.obj_dict.values())
    for o in syncher.obj_dict.values():
        assert hasattr(o, '_found')
        if o._found is True:
            assert o not in deleted_objects
        else:
            assert o in deleted_objects



import json
import re

from django.db.models import Q
from django.contrib.gis.geos import Point, Polygon
from django.contrib.gis.measure import D
from django.contrib.gis.gdal import SRSException, SpatialReference, CoordTransform
from tastypie.http import HttpBadRequest
from tastypie.resources import ModelResource
from tastypie.exceptions import InvalidFilterError, ImmediateHttpResponse
from tastypie.constants import ALL, ALL_WITH_RELATIONS
from tastypie.cache import SimpleCache
from tastypie import fields
from munigeo.models import *
from modeltranslation.translator import translator, NotRegistered

# Use the GPS coordinate system by default
DEFAULT_SRID = 4326

def poly_from_bbox(bbox_val):
    points = bbox_val.split(',')
    if len(points) != 4:
        raise InvalidFilterError("bbox must be in format 'left,bottom,right,top'")
    try:
        points = [float(p) for p in points]
    except ValueError:
        raise InvalidFilterError("bbox values must be floating point or integers")
    poly = Polygon.from_bbox(points)
    return poly

def srid_to_srs(srid):
    if not srid:
        srid = DEFAULT_SRID
    try:
        srid = int(srid)
    except ValueError:
        raise InvalidFilterError("'srid' must be an integer")
    try:
        srs = SpatialReference(srid)
    except SRSException:
        raise InvalidFilterError("SRID %d not found (try 4326 for GPS coordinate system)" % srid)
    return srs

def build_bbox_filter(srid, bbox_val, field_name):
    poly = poly_from_bbox(bbox_val)
    srs = srid_to_srs(srid)
    poly.set_srid(srs.srid)

    if srid != settings.PROJECTION_SRID:
        source_srs = SpatialReference(settings.PROJECTION_SRID)
        ct = CoordTransform(srs, source_srs)
        poly.transform(ct)

    return {"%s__within" % field_name: poly}


LANGUAGES = [x[0] for x in settings.LANGUAGES]

class TranslatableCachedResource(ModelResource):
    def __init__(self, api_name=None):
        super(TranslatableCachedResource, self).__init__(api_name)
        self._meta.cache = SimpleCache(timeout=300)

    def dehydrate(self, bundle):
        bundle = super(TranslatableCachedResource, self).dehydrate(bundle)
        obj = bundle.obj
        try:
            trans_opts = translator.get_options_for_model(type(obj))
        except NotRegistered:
            return bundle

        for field_name in trans_opts.fields.keys():
            if field_name in bundle.data:
                del bundle.data[field_name]

            # Remove the pre-existing data in the bundle.
            for lang in LANGUAGES:
                key = "%s_%s" % (field_name, lang)
                if key in bundle.data:
                    del bundle.data[key]

            d = {}
            default_lang = LANGUAGES[0]
            d[default_lang] = getattr(obj, field_name)
            for lang in LANGUAGES[1:]:
                key = "%s_%s" % (field_name, lang)
                val = getattr(bundle.obj, key, None)
                if val == None:
                    continue
                d[lang] = val

            # If no text provided, leave the field as null
            for key, val in d.items():
                if val != None:
                    break
            else:
                d = None
            bundle.data[field_name] = d

        return bundle
    #_meta.translatable_fields


class AdministrativeDivisionTypeResource(TranslatableCachedResource):
    class Meta:
        queryset = AdministrativeDivisionType.objects.order_by('pk')
        resource_name = 'administrative_division_type'
        filtering = {
            'type': ALL,
            'name': ALL
        }

class AdministrativeDivisionResource(TranslatableCachedResource):
    parent = fields.ForeignKey('munigeo.api.AdministrativeDivisionResource', 'parent', null=True)
    type = fields.ForeignKey(AdministrativeDivisionTypeResource, 'type')

    def _convert_to_geojson(self, bundle):
        obj = bundle.obj
        data = {'type': 'Feature'}
        data['properties'] = bundle.data
        data['id'] = bundle.obj.pk
        geom = obj.geometry.boundary
        data['geometry'] = json.loads(geom.geojson)
        bundle.data = data
        return bundle
    def alter_detail_data_to_serialize(self, request, bundle):
        if request.GET.get('format') == 'geojson':
            return self._convert_to_geojson(bundle)
        return bundle
    def alter_list_data_to_serialize(self, request, bundles):
        if request.GET.get('format') != 'geojson':
            return bundles
        data = {'type': 'FeatureCollection'}
        data['meta'] = bundles['meta']
        data['features'] = [self._convert_to_geojson(bundle) for bundle in bundles['objects']]
        return data
    def apply_filters(self, request, filters):
        obj_list = super(AdministrativeDivisionResource, self).apply_filters(request, filters)
        if request.GET.get('format') == 'geojson':
            obj_list = obj_list.select_related('geometry')
        return obj_list

    def query_to_filters(self, query):
        filters = {}
        filters['name__icontains'] = query
        return filters
    def build_filters(self, filters=None):
        if 'type' in filters:
            type_str = filters['type']
            # If the given type is not digits, assume it's a type name
            if re.match(type_str, r'$[\d]+^'):
                del filters['type']
                filters['type__type'] = type_str

        orm_filters = super(AdministrativeDivisionResource, self).build_filters(filters)
        if filters and 'input' in filters:
            orm_filters.update(self.query_to_filters(filters['input']))
        return orm_filters


    def dehydrate(self, bundle):
        if bundle.request.GET.get('geometry', '').lower() in ('true', '1'):
            srid = bundle.request.GET.get('srid', None)
            srs = srid_to_srs(srid)
            geom = bundle.obj.geometry.boundary
            if srs.srid != geom.srid:
                ct = CoordTransform(geom.srs, srs)
                geom.transform(ct)
            geom_str = geom.geojson
            bundle.data['boundary'] = json.loads(geom_str)

        return super(AdministrativeDivisionResource, self).dehydrate(bundle)

    def determine_format(self, request):
        if request.GET.get('format') == 'geojson':
            return 'application/json'
        return super(AdministrativeDivisionResource, self).determine_format(request)

    class Meta:
        queryset = AdministrativeDivision.objects.order_by('ocd_id').select_related('geometry').select_related('type__type')
        resource_name = 'administrative_division'
        filtering = {
            'type': ALL_WITH_RELATIONS,
            'ocd_id': ALL,
            'begin': ALL,
            'end': ALL,
            'name': ALL,
            'parent': ALL_WITH_RELATIONS,
            'type': ALL_WITH_RELATIONS,
        }
        excludes = ['lft', 'rght', 'tree_id']
        list_allowed_methods = ['get']
        detail_allowed_methods = ['get']


class MunicipalityResource(AdministrativeDivisionResource):
    class Meta:
        queryset = Municipality.objects.all()
        resource_name = 'municipality'
        filtering = {
            'administrative_division': ALL_WITH_RELATIONS,
            'ocd_id': ALL,
            'begin': ALL,
            'end': ALL,
            'name': ALL,
            'parent': ALL_WITH_RELATIONS,
            'type': ALL_WITH_RELATIONS,
        }
        excludes = ['lft', 'rght', 'tree_id']
        list_allowed_methods = ['get']
        detail_allowed_methods = ['get']

class AddressResource(ModelResource):
    municipality = fields.ToOneField('munigeo.api.MunicipalityResource', 'municipality',
        help_text="ID of the municipality that this address belongs to")

    def apply_sorting(self, objects, options=None):
        if options and 'lon' in options and 'lat' in options:
            try:
                lat = float(options['lat'])
                lon = float(options['lon'])
            except ValueError:
                raise InvalidFilterError("'lon' and 'lat' need to be floats")
            pnt = Point(lon, lat, srid=4326)
            pnt.transform(PROJECTION_SRID)
            objects = objects.distance(pnt).order_by('distance')
        return super(AddressResource, self).apply_sorting(objects, options)

    def apply_filters(self, request, applicable_filters):
        if 'distinct_streets' in applicable_filters:
            ds = applicable_filters.pop('distinct_streets')
        else:
            ds = None
        queryset = (super(AddressResource, self)
                    .apply_filters(request, applicable_filters))
        if ds:
            queryset = (queryset.order_by(*ds['order_by'])
                                .distinct(*ds['distinct']))
        return queryset

    def distinct_streets(self, ds):
        filters = {}
        if ds == 'true':
            filters['distinct_streets'] = {
                'order_by': ['municipality', 'street'],
                'distinct': ['municipality', 'street']}
        elif ds != 'false':
            raise ImmediateHttpResponse(
                response=HttpBadRequest(
                    content="{\"error\": \"If given, the option "
                            "distinct_streets must be either 'true' or "
                            "'false'.\"}",
                    content_type='application/json'))
        return filters

    def query_to_filters(self, query):
        filters = {}
        m = re.search(r'(\D+)(\d+)', query, re.U)
        if m:
            number = m.groups()[1]
            street = m.groups()[0].strip()
        else:
            number = None
            street = query.strip()
        filters['street__istartswith'] = street
        if number:
            filters['number'] = int(number)
        return filters

    def build_filters(self, filters=None):
        orm_filters = super(AddressResource, self).build_filters(filters)
        if filters:
            if 'name' in filters:
                orm_filters.update(self.query_to_filters(filters['name']))
            if 'distinct_streets' in filters:
                orm_filters.update(self.distinct_streets(
                    filters['distinct_streets']))
        return orm_filters

    def dehydrate_location(self, bundle):
        loc = bundle.data['location']
        coords = loc['coordinates']
        pnt = Point(coords[0], coords[1], srid=PROJECTION_SRID)
        pnt.transform(4326)
        loc['coordinates'] = [pnt.x, pnt.y]
        return loc

    def dehydrate(self, bundle):
        distance = getattr(bundle.obj, 'distance', None)
        if distance is not None:
            distance = str(distance)
            bundle.data['distance'] = float(distance.strip(' m'))
        bundle.data['name'] = str(bundle.obj)
        return bundle

    class Meta:
        queryset = Address.objects.all()
        filtering = {
            'municipality': ALL,
            'street': ALL,
            'number': ALL,
            'number_end': ALL,
            'letter': ALL,
            'location': ALL,
        }
        list_allowed_methods = ['get']
        detail_allowed_methods = ['get']

class POICategoryResource(TranslatableCachedResource):
    class Meta:
        queryset = POICategory.objects.all()
        filtering = {
            'type': ALL,
            'description': ALL,
        }
        list_allowed_methods = ['get']
        detail_allowed_methods = ['get']

class POIResource(TranslatableCachedResource):
    category = fields.ToOneField(POICategoryResource, 'category')

    def apply_sorting(self, objects, options=None):
        if options and 'lon' in options and 'lat' in options:
            try:
                lat = float(options['lat'])
                lon = float(options['lon'])
            except ValueError:
                raise InvalidFilterError("'lon' and 'lat' need to be floats")
            pnt = Point(lon, lat, srid=4326)
            pnt.transform(PROJECTION_SRID)
            objects = objects.distance(pnt).order_by('distance')
        return super(POIResource, self).apply_sorting(objects, options)

    def dehydrate_location(self, bundle):
        srid = bundle.request.GET.get('srid', None)
        srs = srid_to_srs(srid)
        geom = bundle.obj.location
        if srs.srid != geom.srid:
            ct = CoordTransform(geom.srs, srs)
            geom.transform(ct)
        geom_str = geom.geojson
        return json.loads(geom_str)

    def dehydrate(self, bundle):
        distance = getattr(bundle.obj, 'distance', None)
        if distance is not None:
            distance = str(distance)
            bundle.data['distance'] = float(distance.strip(' m'))
        bundle.data['category_type'] = str(bundle.obj.category.type)
        return bundle

    class Meta:
        queryset = POI.objects.all()
        filtering = {
            'municipality': ALL,
            'category': ALL_WITH_RELATIONS,
        }
        list_allowed_methods = ['get']
        detail_allowed_methods = ['get']


class PlanResource(ModelResource):
    #municipality = fields.ToOneField(MunicipalityResource, 'municipality',
        #help_text="ID of the municipality that this plan belongs to")

    def build_filters(self, filters=None):
        orm_filters = super(PlanResource, self).build_filters(filters)
        if filters and 'bbox' in filters:
            bbox_filter = build_bbox_filter(filters['bbox'], 'geometry')
            orm_filters.update(bbox_filter)
        return orm_filters

    def full_dehydrate(self, bundle, for_list=False):
        # Convert to WGS-84 before outputting.
        bundle.obj.geometry.transform(4326)
        return super(PlanResource, self).full_dehydrate(bundle, for_list)

    class Meta:
        queryset = Plan.objects.all()
        filtering = {
            'municipality': ALL,
            'origin_id': ['exact'],
            'in_effect': ['exact'],
            'geometry': ALL,
        }
        list_allowed_methods = ['get']
        detail_allowed_methods = ['get']

all_resources = [
    AdministrativeDivisionResource, AdministrativeDivisionTypeResource,
    AddressResource, POICategoryResource, POIResource, MunicipalityResource, PlanResource,
]



"""
Open Civic Data helper functions
"""

import re

def make_id(parent=None, **kwargs):
    country = kwargs.pop('country', 'us')
    if len(kwargs) > 1:
        raise ValueError('only one kwarg is allowed for make_id')
    type, type_id = list(kwargs.items())[0]
    if not re.match('^[\w_-]+$', type, re.UNICODE):
        raise ValueError('type must match [\w_]+ [%s]' % type)
    type_id = type_id.lower()
    type_id = re.sub('\.? ', '_', type_id)
    type_id = re.sub('[^\w0-9~_.-]', '~', type_id, re.UNICODE)
    if parent:
        return '{parent}/{type}:{type_id}'.format(parent=parent, type=type,
                                                  type_id=type_id)
    else:
        return 'ocd-division/country:{country}/{type}:{type_id}'.format(
            type=type, type_id=type_id, country=country)



from django.conf import settings


def get_default_srid():
    if hasattr(settings, 'DEFAULT_SRID'):
        srid = settings.DEFAULT_SRID
    elif hasattr(settings, 'PROJECTION_SRID'):
        srid = settings.PROJECTION_SRID
    else:
        # If SRID not configured in settings, use WGS-84.
        srid = 4326

    return srid



# -*- coding: utf-8 -*-
import os
import csv
#import unicodecsv
import requests
import requests_cache
import io
import json

from django.conf import settings
from django import db
from django.contrib.gis.gdal import DataSource, SpatialReference, CoordTransform
from django.contrib.gis.geos import GEOSGeometry, MultiPolygon, Point

from munigeo.models import *
from munigeo.importer.sync import ModelSyncher
from munigeo import ocd

from munigeo.importer.base import Importer, register_importer

POI_LIST = [
    {
        'category': 'park', 'category_desc': 'Park',
        'url': 'http://opendata.manchesterdda.com/data-files/parks_and_open_spaces_20121116.csv',
        'name_col': 0,
        'address_col': 1,
        'desc_col': 5,
        'location_col': 18,
    }, {
        'category': 'library', 'category_desc': 'Library',
        'url': 'http://opendata.manchesterdda.com/data-files/libraries20121113.csv',
        'name_col': 0,
        'address_col': 6, 
        'desc_col': 1,
        'location_col': 20,
    }
]

SERVICE_CATEGORY_MAP = {
    50: ("toilet", "Toilet"),
    9: ("recycling", "Recycling points"),
    #87: ("attractions", "Tourist attractions"),
}

CITADEL_LIST = [
    {
        'url': 'http://www.citadelonthemove.eu/Portals/0/PropertyAgent/517/Files/9/CitadeL-Parking_Lots-Manchester.json',
        'cat_map': {
            'Parking': {
                'category': 'parking',
                'category_desc': 'Parking lot',
            }
        }
    }
]

def convert_from_wgs84(coords):
    pnt = Point(coords[1], coords[0], srid=4326)
    pnt.transform(PROJECTION_SRID)
    return pnt

@register_importer
class ManchesterImporter(Importer):
    name = "manchester"

    def __init__(self, *args, **kwargs):
        super(ManchesterImporter, self).__init__(*args, **kwargs)
        self.data_path = self.options['data_path']
        self.muni_data_path = os.path.join(self.data_path, 'uk', 'manchester')

    def import_municipalities(self):
        muni, c = Municipality.objects.get_or_create(id=44001, name="Manchester")
        self.logger.info("Manchester municipality added.")

    def import_pois_from_csv(self):
        muni = Municipality.objects.get(id=44001)
        for poi_info in POI_LIST:
            self.logger.info("Importing %s" % poi_info['category'])
            self.logger.info(poi_info['url'])
            resp = requests.get(poi_info['url'])
            assert resp.status_code == 200

            cat, c = POICategory.objects.get_or_create(type=poi_info['category'], defaults={'description': poi_info['category_desc']})

            s = resp.text.encode('utf8').decode('utf8')
            f = io.StringIO(s)
            reader = unicodecsv.reader(f, delimiter=',', quotechar='"', encoding='utf8')
            # skip header
            next(reader)

            for idx, row in enumerate(reader):
                origin_id = "man-%s-%d" % (poi_info['category'], idx)
                try:
                    poi = POI.objects.get(origin_id=origin_id)
                except POI.DoesNotExist:
                    poi = POI(origin_id=origin_id)
                poi.name = row[poi_info['name_col']].strip()
                if not poi.name:
                    continue
                coords = row[poi_info['location_col']].strip()
                if not coords:
                    continue
                coords = [float(x) for x in coords.split(',')]
                if coords[0] > 180 or coords[0] < -180:
                    self.logger.info("Skipping invalid coords for %s" % poi.name)
                    continue
                poi.category = cat
                poi.municipality = muni
                poi.location = convert_from_wgs84(coords)
                poi.save()

    def import_pois_from_rest(self):
        URL_BASE = 'http://www.manchester.gov.uk/site/custom_scripts/getServiceDetailsjs.php?service=%d&postcode=M2+5DB&count=10000&format=json'

        muni = Municipality.objects.get(id=44001)
        for srv_id in list(SERVICE_CATEGORY_MAP.keys()):
            cat_type, cat_desc = SERVICE_CATEGORY_MAP[srv_id]
            cat, c = POICategory.objects.get_or_create(type=cat_type, defaults={'description': cat_desc})

            self.logger.info("Importing %s" % cat_type)
            ret = requests.get(URL_BASE % srv_id)
            if ret.status_code != 200:
                raise Exception("HTTP request failed with %d" % ret.status_code)
            # Fix quoting bug
            s = ret.content.replace("\\'", "'")
            ret_json = json.loads(s)
            count = 0
            for srv_info in ret_json:
                srv_id = "man-%s" % str(srv_info['uid'])
                try:
                    poi = POI.objects.get(origin_id=srv_id)
                except POI.DoesNotExist:
                    poi = POI(origin_id=srv_id)
                poi.name = srv_info['name']
                poi.category = cat
                poi.municipality = muni
                if 'address' in srv_info:
                    poi.street_address = srv_info['address']
                else:
                    poi.street_address = ''
                coords = srv_info['latlon'].strip()
                if not coords:
                    continue
                coords = [float(x) for x in coords.split(',')]
                poi.location = convert_from_wgs84(coords)
                poi.save()
                count = count + 1
            self.logger.info("%d imported" % count)

    def import_pois_from_citadel(self):
        muni = Municipality.objects.get(id=44001)
        for d in CITADEL_LIST:
            self._import_citadel(muni, d)

    def import_pois(self):
        requests_cache.install_cache('geo_import_man')
        self.logger.info("Importing POIs from Citadel")
        self.import_pois_from_citadel()
        # self.logger.info("Importing POIs from CSV")
        # self.import_pois_from_csv()
        self.logger.info("Importing POIs from REST")
        self.import_pois_from_rest()



import logging

logger = logging.getLogger(__name__)


class ModelSyncher(object):
    def __init__(self, queryset, generate_obj_id):
        d = {}
        self.generate_obj_id = generate_obj_id
        # Generate a list of all objects
        for obj in queryset:
            d[generate_obj_id(obj)] = obj
            obj._found = False
            obj._changed = False

        self.obj_dict = d

    def mark(self, obj):
        if getattr(obj, '_found', False):
            raise Exception("Object %s (%s) already marked" % (obj, self.generate_obj_id(obj)))

        obj._found = True
        obj_id = self.generate_obj_id(obj)
        if obj_id not in self.obj_dict:
            self.obj_dict[obj_id] = obj
        assert self.obj_dict[obj_id] == obj

    def get(self, obj_id):
        return self.obj_dict.get(obj_id, None)

    def get_deleted_objects(self):
        """Called after an import run, returns objects not found in the source
        system, which need to be deleted from the current system.

        Returns a list of objects.

        Can be used to process related objects or do other
        preprocessing before deleting.

        """
        return [obj for obj in self.obj_dict.values() if not obj._found]

    def finish(self):
        delete_list = self.get_deleted_objects()
        if len(delete_list) > 5 and len(delete_list) > len(self.obj_dict) * 0.4:
            raise Exception("Attempting to delete more than 40% of total items")
        for obj in delete_list:
            logger.debug("Deleting object %s" % obj)
            obj.delete()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('munigeo', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='municipality',
            name='division',
            field=models.OneToOneField(related_name='muni', to='munigeo.AdministrativeDivision', null=True, on_delete=models.CASCADE),
        ),
    ]



# -*- coding: utf-8 -*-

from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext as _
from django.contrib.gis.db import models
from django.db.models.query import QuerySet, Q
from django.conf import settings
from mptt.models import MPTTModel, TreeForeignKey
from mptt.managers import TreeManager

from munigeo.utils import get_default_srid

PROJECTION_SRID = get_default_srid()


@python_2_unicode_compatible
class AdministrativeDivisionType(models.Model):
    type = models.CharField(max_length=60, unique=True, db_index=True,
                            help_text=_("Type name of the division (e.g. muni, school_district)"))
    name = models.CharField(max_length=100,
                            help_text=_("Human-readable name for the division"))
    # European Union Nomenclature des Unités Territoriales Statistiques level
    # nuts_level = models.PositiveSmallIntegerField(null=True, db_index=True)
    # European Union Local Administrative Unit level
    # lau_level = models.PositiveSmallIntegerField(null=True, db_index=True)

    def __str__(self):
        return "%s (%s)" % (self.name, self.type)


class AdministrativeDivisionQuerySet(QuerySet):

    def by_ancestor(self, ancestor):
        manager = self.model.objects
        max_level = manager.determine_max_level()
        qs = Q()
        # Construct an OR'd queryset for each level of parenthood.
        for i in range(max_level):
            key = '__'.join(['parent'] * (i + 1))
            qs |= Q(**{key: ancestor})
        return self.filter(qs)


class AdministrativeDivisionManager(TreeManager):

    def get_queryset(self):
        return AdministrativeDivisionQuerySet(self.model, using=self._db)

    def determine_max_level(self):
        if hasattr(self, '_max_level'):
            return self._max_level
        qs = self.all().order_by('-level')
        # FIXME: Use signals to catch new level being added
        if False and qs.count():
            self._max_level = qs[0].level
        else:
            # Harrison-Stetson method
            self._max_level = 6
        return self._max_level


@python_2_unicode_compatible
class AdministrativeDivision(MPTTModel):
    type = models.ForeignKey(AdministrativeDivisionType, db_index=True, on_delete=models.CASCADE)
    name = models.CharField(max_length=100, null=True, db_index=True)
    parent = TreeForeignKey('self', db_index=True, null=True,
                            related_name='children', on_delete=models.CASCADE)

    origin_id = models.CharField(max_length=50, db_index=True)
    ocd_id = models.CharField(max_length=200, unique=True, db_index=True, null=True,
                              help_text=_("Open Civic Data identifier"))

    municipality = models.ForeignKey('munigeo.Municipality', null=True, on_delete=models.CASCADE)

    # Service districts might have a related service point id
    service_point_id = models.CharField(max_length=50, db_index=True, null=True,
                                        blank=True)

    # Some divisions might be only valid during some time period.
    # (E.g. yearly school districts in Helsinki)
    start = models.DateField(null=True)
    end = models.DateField(null=True)

    modified_at = models.DateTimeField(auto_now=True,
                                       help_text='Time when the information was last changed')

    objects = AdministrativeDivisionManager()

    def __str__(self):
        ocd_id = ''
        if self.ocd_id:
            ocd_id = '%s / ' % self.ocd_id
        return "%s (%s%s)" % (self.name, ocd_id, self.type.type)

    class Meta:
        unique_together = (('origin_id', 'type', 'parent'),)


class AdministrativeDivisionGeometry(models.Model):
    division = models.OneToOneField(AdministrativeDivision, related_name='geometry', on_delete=models.CASCADE)
    boundary = models.MultiPolygonField(srid=PROJECTION_SRID)

    objects = models.GeoManager()


@python_2_unicode_compatible
class Municipality(models.Model):
    id = models.CharField(max_length=100, primary_key=True)
    name = models.CharField(max_length=100, null=True, db_index=True)
    division = models.OneToOneField(AdministrativeDivision, null=True, db_index=True,
                                    related_name='muni', on_delete=models.CASCADE)

    objects = models.Manager()

    def __str__(self):
        return self.name


@python_2_unicode_compatible
class Plan(models.Model):
    municipality = models.ForeignKey(Municipality, on_delete=models.CASCADE)
    geometry = models.MultiPolygonField(srid=PROJECTION_SRID)
    origin_id = models.CharField(max_length=20)
    in_effect = models.BooleanField(default=False)

    objects = models.GeoManager()

    def __str__(self):
        effect = "in effect"
        if not self.in_effect:
            effect = "not " + effect
        return "Plan %s (%s, %s)" % (self.origin_id, self.municipality, effect)

    class Meta:
        unique_together = (('municipality', 'origin_id'),)


@python_2_unicode_compatible
class Street(models.Model):
    name = models.CharField(max_length=100, db_index=True)
    municipality = models.ForeignKey(Municipality, db_index=True, on_delete=models.CASCADE)
    modified_at = models.DateTimeField(auto_now=True,
                                       help_text='Time when the information was last changed')

    def __str__(self):
        return self.name

    class Meta:
        unique_together = (('municipality', 'name'),)


@python_2_unicode_compatible
class Address(models.Model):
    street = models.ForeignKey(Street, db_index=True, related_name='addresses', on_delete=models.CASCADE)
    number = models.CharField(max_length=6, blank=True,
                              help_text="Building number")
    number_end = models.CharField(max_length=6, blank=True,
                                  help_text="Building number end (if range specified)")
    letter = models.CharField(max_length=2, blank=True,
                              help_text="Building letter if applicable")
    location = models.PointField(srid=PROJECTION_SRID,
                                 help_text="Coordinates of the address")

    objects = models.GeoManager()

    modified_at = models.DateTimeField(auto_now=True,
                                       help_text='Time when the information was last changed')

    def __str__(self):
        s = '%s %s' % (self.street, self.number)
        if self.number_end:
            s += '-%s' % self.number_end
        if self.letter:
            s += '%s' % self.letter
        s += ', %s' % self.street.municipality
        return s

    class Meta:
        unique_together = (('street', 'number', 'number_end', 'letter'),)
        ordering = ['street', 'number']


@python_2_unicode_compatible
class Building(models.Model):
    origin_id = models.CharField(max_length=40, db_index=True)
    municipality = models.ForeignKey(Municipality, db_index=True)
    geometry = models.MultiPolygonField(srid=PROJECTION_SRID)

    addresses = models.ManyToManyField(Address, blank=True)

    objects = models.GeoManager()

    modified_at = models.DateTimeField(auto_now=True,
                                       help_text='Time when the information was last changed')

    def __str__(self):
        return '%s in %s' % (self.origin_id, self.municipality)

    class Meta:
        ordering = ['municipality', 'origin_id']


@python_2_unicode_compatible
class POICategory(models.Model):
    type = models.CharField(max_length=50, db_index=True)
    description = models.CharField(max_length=100)

    def __str__(self):
        return "%s (%s)" % (self.type, self.description)


@python_2_unicode_compatible
class POI(models.Model):
    name = models.CharField(max_length=200)
    category = models.ForeignKey(POICategory, db_index=True, on_delete=models.CASCADE)
    description = models.TextField(null=True, blank=True)
    location = models.PointField(srid=PROJECTION_SRID)
    municipality = models.ForeignKey(Municipality, db_index=True, on_delete=models.CASCADE)
    street_address = models.CharField(max_length=100, null=True, blank=True)
    zip_code = models.CharField(max_length=10, null=True, blank=True)
    origin_id = models.CharField(max_length=40, db_index=True, unique=True)

    objects = models.GeoManager()

    def __str__(self):
        return "%s (%s, %s)" % (self.name, self.category.type, self.municipality)



# -*- coding: utf-8 -*-
import os
from optparse import make_option
from django.conf import settings
from django.core.management.base import BaseCommand, CommandError
from django.utils.translation import activate, get_language

from munigeo.importer.base import get_importers


class Command(BaseCommand):
    help = "Import geo data"

    importer_types = ['municipalities', 'divisions', 'addresses', 'pois']

    def add_arguments(self, parser):
        parser.add_argument('module', type=str)
        parser.add_argument('--all', action='store_true', dest='all', help='Import all entities')
        for imp in self.importer_types:
            parser.add_argument('--%s' % imp, dest=imp, action='store_true', help='import %s' % imp)

    def __init__(self):
        super(Command, self).__init__()

    def handle(self, *args, **options):
        importers = get_importers()
        imp_list = ', '.join(sorted(importers.keys()))
        imp_name = options.get('module')
        if not imp_name:
            raise CommandError("Enter the name of the geo importer module. Valid importers: %s" % imp_list)
        if imp_name not in importers:
            raise CommandError("Importer %s not found. Valid importers: %s" % (args[0], imp_list))
        imp_class = importers[imp_name]
        importer = imp_class(options)

        # Activate the default language for the duration of the import
        # to make sure translated fields are populated correctly.
        old_lang = get_language()
        activate(settings.LANGUAGES[0][0])

        for imp_type in self.importer_types:
            name = "import_%s" % imp_type
            method = getattr(importer, name, None)
            if options[imp_type]:
                if not method:
                    raise CommandError("Importer %s does not support importing %s" % (name, imp_type))
            else:
                if not options['all']:
                    continue

            if method:
                method()

        activate(old_lang)



"""
munigeo importer for Helsinki data
"""

import os
import re
import requests
import yaml

from django import db
from datetime import datetime

from django.contrib.gis.gdal import DataSource, SpatialReference, CoordTransform
from django.contrib.gis.gdal.error import GDALException
from django.contrib.gis.geos import GEOSGeometry, MultiPolygon, Point
from django.contrib.gis import gdal

from munigeo.models import *
from munigeo.importer.sync import ModelSyncher
from munigeo import ocd

from munigeo.importer.base import Importer, register_importer

MUNI_URL = "http://tilastokeskus.fi/meta/luokitukset/kunta/001-2013/tekstitiedosto.txt"

# The Finnish national grid coordinates in TM35-FIN according to JHS-180
# specification. We use it as a bounding box.
FIN_GRID = [-548576, 6291456, 1548576, 8388608]
TM35_SRID = 3067

SERVICE_CATEGORY_MAP = {
    25480: ("library", "Library"),
    28148: ("swimming_pool", "Swimming pool"),
    25402: ("toilet", "Toilet"),
    25344: ("recycling", "Recycling point"),
    25664: ("park", "Park"),
}


GK25_SRID = 3879
GK25_SRS = SpatialReference(GK25_SRID)
PROJECTION_SRS = SpatialReference(PROJECTION_SRID)
WEB_MERCATOR_SRS = SpatialReference(3857)

coord_transform = None
if GK25_SRS.srid != PROJECTION_SRID:
    target_srs = SpatialReference(PROJECTION_SRID)
    coord_transform = CoordTransform(GK25_SRS, target_srs)

def convert_from_gk25(north, east):
    ps = "POINT (%f %f)" % (east, north)
    g = gdal.OGRGeometry(ps, GK25_SRS)
    if coord_transform:
        g.transform(coord_transform)
    return g

    pnt = Point(east, north, srid=GK25_SRID)
    if PROJECTION_SRID == GK25_SRID:
        return pnt
    pnt.transform(coord_transform)
    return pnt

def poly_diff(p1, p2):
    # Make sure we calculate the area with a 2d coordinate system
    if p1.srs.units[1] == 'degree':
        tf = CoordTransform(p1.srs, WEB_MERCATOR_SRS)
        p1 = p1.clone()
        p1.transform(tf)
        p2 = p2.clone()
        p2.transform(tf)
    return (p1 - p2).area


@register_importer
class HelsinkiImporter(Importer):
    name = "helsinki"

    def __init__(self, *args, **kwargs):
        super(HelsinkiImporter, self).__init__(*args, **kwargs)
        self.muni_data_path = 'fi/helsinki'

    def _find_parent_division(self, parent_info):
        args = {
            'type__type': parent_info['type'], 
            'origin_id': parent_info['id'],
            'parent__parent': parent_info['parent']
        }
        return AdministrativeDivision.objects.get(**args)

    def _import_division(self, muni, div, type_obj, syncher, parent_dict, feat):
        #
        # Geometry
        #
        geom = feat.geom
        if not geom.srid:
            geom.srid = GK25_SRID
        if geom.srid != PROJECTION_SRID:
            ct = CoordTransform(SpatialReference(geom.srid), SpatialReference(PROJECTION_SRID))
            geom.transform(ct)
        # geom = geom.geos.intersection(parent.geometry.boundary)
        geom = geom.geos
        if geom.geom_type == 'Polygon':
            geom = MultiPolygon(geom, srid=geom.srid)

        #
        # Attributes
        #
        attr_dict = {}
        lang_dict = {}
        for attr, field in div['fields'].items():
            if isinstance(field, dict):
                # Languages
                d = {}
                for lang, field_name in field.items():
                    val = feat[field_name].as_string()
                    # If the name is in all caps, fix capitalization.
                    if not re.search('[a-z]', val):
                        val = val.title()
                    d[lang] = val.strip()
                lang_dict[attr] = d
            else:
                val = feat[field].as_string()
                attr_dict[attr] = val.strip()

        origin_id = attr_dict['origin_id']
        del attr_dict['origin_id']

        if 'parent' in div:
            if 'parent_id' in attr_dict:
                parent = parent_dict[attr_dict['parent_id']]
                del attr_dict['parent_id']
            else:
                # If no parent id is available, we determine the parent
                # heuristically by choosing the one that we overlap with
                # the most.
                parents = []
                for parent in parent_dict.values():
                    diff_area = poly_diff(geom, parent.geometry.boundary)
                    if diff_area < 300:
                        parents.append(parent)
                if not parents:
                    raise Exception("No parent found for %s" % origin_id)
                elif len(parents) > 1:
                    raise Exception("Too many parents for %s" % origin_id)
                parent = parents[0]
        elif 'parent_ocd_id' in div:
            try:
                parent = AdministrativeDivision.objects.get(ocd_id=div['parent_ocd_id'])
            except AdministrativeDivision.DoesNotExist:
                parent = None
        else:
            parent = muni.division

        if 'parent' in div and parent:
            full_id = "%s-%s" % (parent.origin_id, origin_id)
        else:
            full_id = origin_id
        obj = syncher.get(full_id)
        if not obj:
            obj = AdministrativeDivision(origin_id=origin_id, type=type_obj)

        validity_time_period = div.get('validity')
        if validity_time_period:
            obj.start = validity_time_period.get('start')
            obj.end = validity_time_period.get('end')
            if obj.start:
                obj.start = datetime.strptime(obj.start, '%Y-%m-%d').date()
            if obj.end:
                obj.end = datetime.strptime(obj.end, '%Y-%m-%d').date()

        if div.get('no_parent_division', False):
            muni = None

        obj.parent = parent
        obj.municipality = muni

        for attr in attr_dict.keys():
            setattr(obj, attr, attr_dict[attr])
        for attr in lang_dict.keys():
            for lang, val in lang_dict[attr].items():
                key = "%s_%s" % (attr, lang)
                setattr(obj, key, val)

        if 'ocd_id' in div:
            assert (parent and parent.ocd_id) or 'parent_ocd_id' in div
            if parent:
                if div.get('parent_in_ocd_id', False):
                    args = {'parent': parent.ocd_id}
                else:
                    args = {'parent': muni.division.ocd_id}
            else:
                args = {'parent': div['parent_ocd_id']}
            val = attr_dict['ocd_id']
            args[div['ocd_id']] = val
            obj.ocd_id = ocd.make_id(**args)
            self.logger.debug("%s" % obj.ocd_id)
        obj.save()
        syncher.mark(obj)

        try:
            geom_obj = obj.geometry
        except AdministrativeDivisionGeometry.DoesNotExist:
            geom_obj = AdministrativeDivisionGeometry(division=obj)

        geom_obj.boundary = geom
        geom_obj.save()

    @db.transaction.atomic
    def _import_one_division_type(self, muni, div):
        def make_div_id(obj):
            if 'parent' in div:
                return "%s-%s" % (obj.parent.origin_id, obj.origin_id)
            else:
                return obj.origin_id

        self.logger.info(div['name'])
        if not 'origin_id' in div['fields']:
            raise Exception("Field 'origin_id' not defined in config section '%s'" % div['name'])
        try:
            type_obj = AdministrativeDivisionType.objects.get(type=div['type'])
        except AdministrativeDivisionType.DoesNotExist:
            type_obj = AdministrativeDivisionType(type=div['type'])
            type_obj.name = div['name']
            type_obj.save()

        div_qs = AdministrativeDivision.objects.filter(type=type_obj)
        if not div.get('no_parent_division', False):
            div_qs = div_qs.by_ancestor(muni.division).select_related('parent')
        syncher = ModelSyncher(div_qs, make_div_id)

        # Cache the list of possible parents. Assumes parents are imported
        # first.
        if 'parent' in div:
            parent_list = AdministrativeDivision.objects.\
                filter(type__type=div['parent']).by_ancestor(muni.division)
            parent_dict = {}
            for o in parent_list:
                assert o.origin_id not in parent_dict
                parent_dict[o.origin_id] = o
        else:
            parent_dict = None

        if 'file' in div:
            path = self.find_data_file(os.path.join(self.division_data_path, div['file']))
            ds = DataSource(path, encoding='iso8859-1')
        else:
            wfs_url = 'WFS:' + div['wfs_url']
            if '?' in wfs_url:
                sep = '&'
            else:
                sep = '?'
            url = wfs_url + sep + 'typeName=' + div['wfs_layer'] + '&' + "srsName=EPSG:%d" % PROJECTION_SRID + '&' + "outputFormat=application/json"
            ds = DataSource(url)
        lyr = ds[0]
        assert len(ds) == 1
        with AdministrativeDivision.objects.delay_mptt_updates():
            for feat in lyr:
                self._import_division(muni, div, type_obj, syncher, parent_dict, feat)

    def import_divisions(self):
        path = self.find_data_file(os.path.join(self.muni_data_path, 'config.yml'))
        config = yaml.safe_load(open(path, 'r', encoding='utf-8'))
        self.division_data_path = os.path.join(self.muni_data_path, config['paths']['division'])

        muni = Municipality.objects.get(division__origin_id=config['origin_id'])
        self.muni = muni
        for div in config['divisions']:
            try:
                self._import_one_division_type(muni, div)
            except GDALException as e:
                self.logger.warning('Skipping division %s : %s' % (div, e))

    def _import_plans(self, fname, in_effect):
        path = os.path.join(self.data_path, 'kaavahakemisto', fname)
        ds = DataSource(path, encoding='iso8859-1')
        lyr = ds[0]

        for idx, feat in enumerate(lyr):
            origin_id = feat['kaavatunnus'].as_string()
            geom = feat.geom
            geom.srid = GK25_SRID
            geom.transform(PROJECTION_SRID)
            if origin_id not in self.plan_map:
                obj = Plan(origin_id=origin_id, municipality=self.muni)
                self.plan_map[origin_id] = obj
            else:
                obj = self.plan_map[origin_id]
                if not obj.found:
                    obj.geometry = None
            poly = GEOSGeometry(geom.wkb, srid=geom.srid)
            if obj.geometry:
                obj.geometry.append(poly)
            else:
                obj.geometry = MultiPolygon(poly)
            obj.in_effect = in_effect
            obj.found = True
            if (idx % 500) == 0:
                self.logger.info("%d imported" % idx)
        if in_effect:
            type_str = "in effect"
        else:
            type_str = "development"
        self.logger.info("%d %s plans imported" % (idx, type_str))

    def import_plans(self):
        self.plan_map = {}
        self.muni = Municipality.objects.get(name="Helsinki")
        for obj in Plan.objects.filter(municipality=self.muni):
            self.plan_map[obj.origin_id] = obj
            obj.found = False
        self._import_plans('Lv_rajaus.TAB', True)
        self._import_plans('Kaava_vir_rajaus.TAB', False)
        self.logger.info("Saving")
        for key, obj in self.plan_map.items():
            if obj.found:
                obj.save()
            else:
                self.logger.info("Plan %s deleted" % obj.name)

    @db.transaction.atomic
    def import_addresses(self):

        wfs_url = 'https://kartta.hel.fi/ws/geoserver/avoindata/wfs?SERVICE=WFS&VERSION=1.0.0&REQUEST=GetFeature&TYPENAME=avoindata:PKS_osoiteluettelo&SRSNAME=EPSG:3067'
        self.logger.info("Loading master data from WFS datasource")
        ds = DataSource(wfs_url)
        lyr = ds[0]
        assert len(ds) == 1

        muni_names = ('Helsinki', 'Espoo', 'Vantaa', 'Kauniainen')
        muni_list = Municipality.objects.filter(name_fi__in=muni_names)
        muni_dict = {}

        def make_addr_id(num, num_end, letter):
            if num_end is None:
                num_end = ''
            if letter is None:
                letter = ''
            return '%s-%s-%s' % (num, num_end, letter)

        for muni in muni_list:
            muni_dict[muni.name_fi] = muni

            self.logger.info("Loading existing data for {}".format(muni))

            streets = Street.objects.filter(municipality=muni)
            muni.streets_by_name = {}
            muni.streets_by_id = {}
            for s in streets:
                muni.streets_by_name[s.name_fi] = s
                muni.streets_by_id[s.id] = s
                s.addrs = {}
                s._found = False

            addr_list = Address.objects.filter(street__municipality=muni)
            for a in addr_list:
                a._found = False
                street = muni.streets_by_id[a.street_id]
                street.addrs[make_addr_id(a.number, a.number_end, a.letter)] = a

        bulk_addr_list = []
        bulk_street_list = []
        count = 0

        self.logger.info("starting data synchronization")
        for feat in lyr:
            count += 1
            if count % 1000 == 0:
                self.logger.debug("{} processed".format(count))

            street_name = feat.get('katunimi').strip()
            street_name_sv = feat.get('gatan').strip()
            num = feat.get('osoitenumero')

            if not num:
                self.logger.debug("Rejecting {}, due to {} not being valid street number".format(street_name, num))
                continue
            else:
                if num == '0':
                    self.logger.debug("Rejecting {}, due to {} not being valid street number".format(street_name, num))
                    continue

            num2 = feat.get('osoitenumero2')
            if num2 == 0:
                num2 = ''
            letter = feat.get('osoitekirjain').strip()
            coord_n = int(feat.get('n'))
            coord_e = int(feat.get('e'))
            muni_name = feat.get('kaupunki')

            muni = muni_dict[muni_name]
            street = muni.streets_by_name.get(street_name, None)
            if not street:
                self.logger.info("street {} not found in DB, creating it".format(street_name))
                street = Street(name_fi=street_name, name=street_name, municipality=muni)
                street.name_sv = street_name_sv

                #bulk_street_list.append(street)
                street.save()
                muni.streets_by_name[street_name] = street
                street.addrs = {}
            else:
                if street.name_sv != street_name_sv:
                    self.logger.warning("{}: {} -> {}".format(street, street.name_sv, street_name_sv))
                    street.name_sv = street_name_sv
                    street.save()
            street._found = True

            addr_id = make_addr_id(num, num2, letter)
            addr = street.addrs.get(addr_id, None)
            location = convert_from_gk25(coord_n, coord_e)
            if not addr:
                self.logger.debug("Street {} did not have address {}. Creating".format(street.name, addr_id))
                addr = Address(street=street, number=num, number_end=num2, letter=letter)
                addr.location = location.wkb
                bulk_addr_list.append(addr)
                street.addrs[addr_id] = addr
            else:
                if addr._found:
                    self.logger.debug("{}: is duplicate, skipping".format(addr))
                    continue
                # if the location has changed for more than 10cm, save the new one.
                assert addr.location.srid == location.srid, "SRID changed"
                #if addr.location.distance(location) >= 0.10:
                #    self.logger.info("%s: Location changed" % addr)
                #    addr.location = location
                #    addr.save()
            addr._found = True

            # self.logger.info("%s: %s %d%s N%d E%d (%f,%f)" % (muni_name, street, num, letter, coord_n, coord_e, pnt.y, pnt.x))

            if len(bulk_addr_list) >= 10000:
                self.logger.info("Saving %d new addresses" % len(bulk_addr_list))

                Address.objects.bulk_create(bulk_addr_list)
                bulk_addr_list = []

                # Reset DB query store to free up memory
                db.reset_queries()

        if bulk_addr_list:
            self.logger.info("Saving {} new addresses".format(len(bulk_addr_list)))
            Address.objects.bulk_create(bulk_addr_list)
            bulk_addr_list = []

        for muni in muni_list:
            for s in muni.streets_by_name.values():
                if not s._found:
                    self.logger.info("Street {} removed".format(s))
                    s.delete()
                    continue
                for a in s.addrs.values():
                    if not a._found:
                        self.logger.info("Address {} removed".format(a))
                        a.delete()

        self.logger.info("synchronization complete")

    def import_pois(self):
        URL_BASE = 'http://www.hel.fi/palvelukarttaws/rest/v2/unit/?service=%d'

        muni_dict = {}
        for muni in Municipality.objects.all():
            muni_dict[muni.name] = muni

        for srv_id in list(SERVICE_CATEGORY_MAP.keys()):
            cat_type, cat_desc = SERVICE_CATEGORY_MAP[srv_id]
            cat, c = POICategory.objects.get_or_create(type=cat_type, defaults={'description': cat_desc})

            self.logger.info("Importing %s" % cat_type)
            ret = requests.get(URL_BASE % srv_id)
            for srv_info in ret.json():
                srv_id = str(srv_info['id'])
                try:
                    poi = POI.objects.get(origin_id=srv_id)
                except POI.DoesNotExist:
                    poi = POI(origin_id=srv_id)
                poi.name = srv_info['name_fi']
                poi.category = cat
                if not 'address_city_fi' in srv_info:
                    self.logger.info("No city!")
                    self.logger.info(srv_info)
                    continue
                city_name = srv_info['address_city_fi']
                if not city_name in muni_dict:
                    city_name = city_name.encode('utf8')
                    post_code = srv_info.get('address_zip', '')
                    if post_code.startswith('00'):
                        self.logger.info("%s: %s (%s)" % (srv_info['id'], poi.name.encode('utf8'), city_name))
                        city_name = "Helsinki"
                    elif post_code.startswith('01'):
                        self.logger.info("%s: %s (%s)" % (srv_info['id'], poi.name.encode('utf8'), city_name))
                        city_name = "Vantaa"
                    elif post_code in ('02700', '02701', '02760'):
                        self.logger.info("%s: %s (%s)" % (srv_info['id'], poi.name.encode('utf8'), city_name))
                        city_name = "Kauniainen"
                    elif post_code.startswith('02'):
                        self.logger.info("%s: %s (%s)" % (srv_info['id'], poi.name.encode('utf8'), city_name))
                        city_name = "Espoo"
                    else:
                        self.logger.info(srv_info)
                poi.municipality = muni_dict[city_name]
                poi.street_address = srv_info.get('street_address_fi', None)
                poi.zip_code = srv_info.get('address_zip', None)
                if not 'northing_etrs_gk25' in srv_info:
                    self.logger.info("No location!")
                    self.logger.info(srv_info)
                    continue
                poi.location = convert_from_gk25(srv_info['northing_etrs_gk25'], srv_info['easting_etrs_gk25'])
                poi.save()



import os
import requests
import json
import logging
from django.utils.text import slugify
from django.contrib.gis.gdal import DataSource, SpatialReference, CoordTransform
from django.contrib.gis.geos import GEOSGeometry, MultiPolygon, Point
from django.conf import settings

from munigeo.models import *
from munigeo.importer.sync import ModelSyncher

def convert_from_wgs84(coords):
    pnt = Point(coords[1], coords[0], srid=4326)
    pnt.transform(PROJECTION_SRID)
    return pnt

class Importer(object):
    def _import_citadel(self, muni, info):
        muni_slug = slugify(muni.name)

        self.logger.info("Importing from Citadel")
        resp = requests.get(info['url'])
        assert resp.status_code == 200
        s = resp.content.decode('utf8')
        resp_json = json.loads(s)

        for d in resp_json['dataset']['poi']:
            citadel_type = d['category'][0]
            cat_info = info['cat_map'][citadel_type]
            cat, _ = POICategory.objects.get_or_create(type=cat_info['category'],
                    defaults={'description': cat_info['category_desc']})

            origin_id = "%s-%s-%s" % (muni_slug, cat.type, d['id'])

            try:
                poi = POI.objects.get(origin_id=origin_id)
            except POI.DoesNotExist:
                poi = POI(origin_id=origin_id)
            poi.name = d['title'].strip()
            coords = d['location']['point']['pos']['posList']
            if not coords:
                continue
            coords = [float(x) for x in coords.split(' ')]
            if coords[0] > 180 or coords[0] < -180:
                self.logger.info("Skipping invalid coords for %s" % poi.name)
                continue
            poi.category = cat
            poi.municipality = muni
            poi.location = convert_from_wgs84(coords)
            poi.save()
            self.logger.info(poi)

    def find_data_file(self, data_file):
        for path in self.data_paths:
            full_path = os.path.join(path, data_file)
            if os.path.exists(full_path):
                return full_path
        raise FileNotFoundError("Data file '%s' not found" % data_file)

    def __init__(self, options):
        self.logger = logging.getLogger("%s_importer" % self.name)

        if hasattr(settings, 'PROJECT_ROOT'):
            root_dir = settings.PROJECT_ROOT
        else:
            root_dir = settings.BASE_DIR
        self.data_paths = [os.path.join(root_dir, 'data')]
        module_path = os.path.dirname(__file__)
        app_path = os.path.abspath(os.path.join(module_path, '..', 'data'))
        self.data_paths.append(app_path)

        self.options = options

importers = {}

def register_importer(klass):
    importers[klass.name] = klass
    return klass

def get_importers():
    if importers:
        return importers
    module_path = __name__.rpartition('.')[0]
    # Importing the packages will cause their register_importer() methods
    # being called.
    for fname in os.listdir(os.path.dirname(__file__)):
        module, ext = os.path.splitext(fname)
        if ext.lower() != '.py':
            continue
        if module in ('__init__', 'base'):
            continue
        full_path = "%s.%s" % (module_path, module)
        ret = __import__(full_path, locals(), globals())
    return importers



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import django.contrib.gis.db.models.fields
import mptt.fields

from munigeo.utils import get_default_srid

DEFAULT_SRID = get_default_srid()


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Address',
            fields=[
                ('id', models.AutoField(auto_created=True, serialize=False, verbose_name='ID', primary_key=True)),
                ('number', models.CharField(max_length=6, blank=True, help_text='Building number')),
                ('number_end', models.CharField(max_length=6, blank=True, help_text='Building number end (if range specified)')),
                ('letter', models.CharField(max_length=2, blank=True, help_text='Building letter if applicable')),
                ('location', django.contrib.gis.db.models.fields.PointField(srid=DEFAULT_SRID, help_text='Coordinates of the address')),
            ],
            options={
                'ordering': ['street', 'number'],
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='AdministrativeDivision',
            fields=[
                ('id', models.AutoField(auto_created=True, serialize=False, verbose_name='ID', primary_key=True)),
                ('name', models.CharField(null=True, max_length=100, db_index=True)),
                ('name_fi', models.CharField(null=True, max_length=100, db_index=True)),
                ('name_sv', models.CharField(null=True, max_length=100, db_index=True)),
                ('name_en', models.CharField(null=True, max_length=100, db_index=True)),
                ('origin_id', models.CharField(max_length=50, db_index=True)),
                ('ocd_id', models.CharField(null=True, max_length=200, unique=True, db_index=True, help_text='Open Civic Data identifier')),
                ('service_point_id', models.CharField(null=True, max_length=50, blank=True, db_index=True)),
                ('start', models.DateField(null=True)),
                ('end', models.DateField(null=True)),
                ('modified_at', models.DateTimeField(auto_now=True, help_text='Time when the information was last changed')),
                ('lft', models.PositiveIntegerField(editable=False, db_index=True)),
                ('rght', models.PositiveIntegerField(editable=False, db_index=True)),
                ('tree_id', models.PositiveIntegerField(editable=False, db_index=True)),
                ('level', models.PositiveIntegerField(editable=False, db_index=True)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='AdministrativeDivisionGeometry',
            fields=[
                ('id', models.AutoField(auto_created=True, serialize=False, verbose_name='ID', primary_key=True)),
                ('boundary', django.contrib.gis.db.models.fields.MultiPolygonField(srid=DEFAULT_SRID)),
                ('division', models.OneToOneField(to='munigeo.AdministrativeDivision', related_name='geometry', on_delete=models.CASCADE)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='AdministrativeDivisionType',
            fields=[
                ('id', models.AutoField(auto_created=True, serialize=False, verbose_name='ID', primary_key=True)),
                ('type', models.CharField(max_length=60, unique=True, db_index=True, help_text='Type name of the division (e.g. muni, school_district)')),
                ('name', models.CharField(max_length=100, help_text='Human-readable name for the division')),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Municipality',
            fields=[
                ('id', models.CharField(max_length=100, serialize=False, primary_key=True)),
                ('name', models.CharField(null=True, max_length=100, db_index=True)),
                ('name_fi', models.CharField(null=True, max_length=100, db_index=True)),
                ('name_sv', models.CharField(null=True, max_length=100, db_index=True)),
                ('name_en', models.CharField(null=True, max_length=100, db_index=True)),
                ('division', models.ForeignKey(to='munigeo.AdministrativeDivision', null=True, unique=True, related_name='muni', on_delete=models.CASCADE)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Plan',
            fields=[
                ('id', models.AutoField(auto_created=True, serialize=False, verbose_name='ID', primary_key=True)),
                ('geometry', django.contrib.gis.db.models.fields.MultiPolygonField(srid=DEFAULT_SRID)),
                ('origin_id', models.CharField(max_length=20)),
                ('in_effect', models.BooleanField(default=False)),
                ('municipality', models.ForeignKey(to='munigeo.Municipality', on_delete=models.CASCADE)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='POI',
            fields=[
                ('id', models.AutoField(auto_created=True, serialize=False, verbose_name='ID', primary_key=True)),
                ('name', models.CharField(max_length=200)),
                ('description', models.TextField(null=True, blank=True)),
                ('location', django.contrib.gis.db.models.fields.PointField(srid=DEFAULT_SRID)),
                ('street_address', models.CharField(null=True, max_length=100, blank=True)),
                ('zip_code', models.CharField(null=True, max_length=10, blank=True)),
                ('origin_id', models.CharField(max_length=40, unique=True, db_index=True)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='POICategory',
            fields=[
                ('id', models.AutoField(auto_created=True, serialize=False, verbose_name='ID', primary_key=True)),
                ('type', models.CharField(max_length=50, db_index=True)),
                ('description', models.CharField(max_length=100)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Street',
            fields=[
                ('id', models.AutoField(auto_created=True, serialize=False, verbose_name='ID', primary_key=True)),
                ('name', models.CharField(max_length=100, db_index=True)),
                ('name_fi', models.CharField(null=True, max_length=100, db_index=True)),
                ('name_sv', models.CharField(null=True, max_length=100, db_index=True)),
                ('name_en', models.CharField(null=True, max_length=100, db_index=True)),
                ('municipality', models.ForeignKey(to='munigeo.Municipality', on_delete=models.CASCADE)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.AlterUniqueTogether(
            name='street',
            unique_together=set([('municipality', 'name')]),
        ),
        migrations.AddField(
            model_name='poi',
            name='category',
            field=models.ForeignKey(to='munigeo.POICategory', on_delete=models.CASCADE),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='poi',
            name='municipality',
            field=models.ForeignKey(to='munigeo.Municipality', on_delete=models.CASCADE),
            preserve_default=True,
        ),
        migrations.AlterUniqueTogether(
            name='plan',
            unique_together=set([('municipality', 'origin_id')]),
        ),
        migrations.AddField(
            model_name='administrativedivision',
            name='municipality',
            field=models.ForeignKey(null=True, to='munigeo.Municipality', on_delete=models.CASCADE),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='administrativedivision',
            name='parent',
            field=mptt.fields.TreeForeignKey(to='munigeo.AdministrativeDivision', related_name='children', null=True, on_delete=models.CASCADE),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='administrativedivision',
            name='type',
            field=models.ForeignKey(to='munigeo.AdministrativeDivisionType', on_delete=models.CASCADE),
            preserve_default=True,
        ),
        migrations.AlterUniqueTogether(
            name='administrativedivision',
            unique_together=set([('origin_id', 'type', 'parent')]),
        ),
        migrations.AddField(
            model_name='address',
            name='street',
            field=models.ForeignKey(related_name='addresses', to='munigeo.Street', on_delete=models.CASCADE),
            preserve_default=True,
        ),
        migrations.AlterUniqueTogether(
            name='address',
            unique_together=set([('street', 'number', 'number_end', 'letter')]),
        ),
    ]



import django
from django.conf import settings

CUSTOM_INSTALLED_APPS = (
    'munigeo',
)

ALWAYS_INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
)

ALWAYS_MIDDLEWARE_CLASSES = (
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
)
settings.configure(
    DATABASES={
        'default': {
            'ENGINE': 'django.contrib.gis.db.backends.postgis',
            'NAME': 'servicemap-api',
            'ATOMIC_REQUESTS': True,
        }
    },
    SECRET_KEY="django_tests_secret_key",
    DEBUG=False,
    TEMPLATE_DEBUG=False,
    ALLOWED_HOSTS=[],
    INSTALLED_APPS=ALWAYS_INSTALLED_APPS + CUSTOM_INSTALLED_APPS,
    MIDDLEWARE_CLASSES=ALWAYS_MIDDLEWARE_CLASSES,
    LANGUAGE_CODE='en-us',
    TIME_ZONE='UTC',
    USE_I18N=True,
    USE_L10N=True,
    USE_TZ=True,
    STATIC_URL='/static/',
)

django.setup()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
from setuptools import setup

README = open(os.path.join(os.path.dirname(__file__), 'README.md')).read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-munigeo',
    version='0.2.21',
    packages=['munigeo'],
    include_package_data=True,
    license='BSD License',
    description='A Django app for processing municipality-related geospatial data.',
    long_description=README,
    url='https://github.com/City-of-Helsinki/django-munigeo',
    author='Juha Yrjölä',
    author_email='juha.yrjola@iki.fi',
    install_requires=[
        'Django',
        'requests',
        'requests_cache',
        'django_mptt',
        'django_modeltranslation',
        'six',
        'pyyaml',
    ],
    setup_requires=['pytest-runner'],
    tests_require=['pytest', 'pytest-django'],
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Scientific/Engineering :: GIS',
    ],
)



import json
import requests
from django.http import HttpResponse, HttpResponseBadRequest, HttpResponseNotAllowed, HttpResponseNotFound
from django.conf import settings

GOOGLE_URL_BASE = "https://maps.googleapis.com/maps/api/place/"

def google_autocomplete(request):
    if request.method != 'GET':
        return HttpResponseNotAllowed(['GET'])
    query = request.GET.get('query', '').strip()
    if not query:
        return HttpResponseBadRequest()
    callback = request.GET.get('callback', None)

    if not hasattr(settings, 'GOOGLE_API_KEY'):
        return HttpResponseNotFound("No API key configured")
    language = request.GET.get('language', 'en')
    args = {'key': settings.GOOGLE_API_KEY, 'language': language, 'sensor': 'true'}
    args['input'] = query
    args['types'] = 'geocode'
    if 'country' in request.GET:
        args['components'] = 'country:%s' % (request.GET['country'])

    r = requests.get(GOOGLE_URL_BASE + 'autocomplete/json', params=args)
    if r.status_code != 200:
        return HttpResponseBadRequest()
    ret = r.json()
    s = json.dumps(ret)
    if callback:
        s = '%s && %s(%s);' % (callback, callback, s)
    return HttpResponse(s, content_type="application/json")

def google_details(request):
    if request.method != 'GET':
        return HttpResponseNotAllowed(['GET'])
    ref = request.GET.get('reference', '').strip()
    if not ref:
        return HttpResponseBadRequest()
    callback = request.GET.get('callback', None)

    if not hasattr(settings, 'GOOGLE_API_KEY'):
        return HttpResponseNotFound("No API key configured")
    language = request.GET.get('language', 'en')
    args = {'key': settings.GOOGLE_API_KEY, 'language': language, 'sensor': 'true'}
    args['reference'] = ref

    r = requests.get(GOOGLE_URL_BASE + 'details/json', params=args)
    if r.status_code != 200:
        return HttpResponseBadRequest()
    ret = r.json()
    s = json.dumps(ret)
    if callback:
        s = '%s && %s(%s);' % (callback, callback, s)
    return HttpResponse(s, content_type="application/json")



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('munigeo', '0002_auto_20150608_1607'),
    ]

    operations = [
        migrations.AddField(
            model_name='address',
            name='modified_at',
            field=models.DateTimeField(help_text='Time when the information was last changed', auto_now=True),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='street',
            name='modified_at',
            field=models.DateTimeField(help_text='Time when the information was last changed', auto_now=True),
            preserve_default=False,
        ),
    ]
